@@ -13,10 +13,8 @@
 
 import os
 import sys
-from pygments.formatters import HtmlFormatter
-from pygments.styles import get_all_styles
 
-# Sphinx-automodapi requires proplot on path
+# Add proplot to path for sphinx-automodapi
 sys.path.insert(0, os.path.abspath('..'))
 
 # Add docs folder to PATH for local 'sphinxext' extensions
@@ -184,6 +182,8 @@ pygments_style = 'none'
 # Create local pygments copies
 # Previously used: https://github.com/richleland/pygments-css
 # But do not want to depend on some random repository
+from pygments.formatters import HtmlFormatter  # noqa: E402
+from pygments.styles import get_all_styles  # noqa: E402
 path = os.path.join('_static', 'pygments')
 if not os.path.isdir(path):
     os.mkdir(path)
@@ -194,6 +194,10 @@ for style in get_all_styles():
     with open(path, 'w') as f:
         f.write(HtmlFormatter(style=style).get_style_defs('.highlight'))
 
+# Create sample .proplotrc file
+from proplot.rctools import _write_defaults  # noqa: E402
+_write_defaults(os.path.join('_static', 'proplotrc'), comment=False)
+
 # Role
 # default family is py, but can also set default role so don't need
 # :func:`name`, :module:`name`, etc.
